/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.user;

import java.util.Collection;
import java.util.Date;

import org.bson.codecs.pojo.annotations.BsonId;
import org.bson.types.ObjectId;

import com.fasterxml.jackson.annotation.JsonIgnore;

import de.ozgcloud.user.settings.UserSettings;
import io.quarkus.runtime.annotations.RegisterForReflection;
import lombok.AllArgsConstructor;
import lombok.Builder;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.Singular;
import lombok.ToString;

@Builder(toBuilder = true)
@NoArgsConstructor
@AllArgsConstructor
@Getter
@Setter
@ToString
@RegisterForReflection
public class User {

	public static final String EXTERNAL_ID_FIELD = "externalId";
	public static final String KEYCLOAK_USER_ID = "keycloakUserId";
	public static final String EMAIL_FIELD = "email";
	public static final String DELETED_FIELD = "deleted";
	public static final String LAST_SYNC_TIMESTAMP_FIELD = "lastSyncTimestamp";
	public static final String LASTNAME_FIELD = "lastName";
	public static final String FIRSTNAME_FIELD = "firstName";
	public static final String FULL_NAME_FIELD = "fullName";
	public static final String FULL_NAME_REVERSED_FIELD = "fullNameReversed";
	public static final String USERNAME_FIELD = "username";
	public static final String ORGANISATIONS_EINHEIT_IDS_FIELD = "organisationsEinheitIds";
	public static final String NOTIFICATION_SEND_FOR_FIELD = "userSettings.notificationsSendFor";

	@JsonIgnore
	@BsonId
	private ObjectId id;
	@JsonIgnore
	private String externalId;
	@JsonIgnore
	private String keycloakUserId;
	private String firstName;
	private String lastName;
	@JsonIgnore
	private String fullName;
	@JsonIgnore
	private String fullNameReversed;
	private String username;
	private String email;
	@JsonIgnore
	private Collection<String> roles;
	@JsonIgnore
	@Singular
	private Collection<String> organisationsEinheitIds;
	private Date createdAt;
	@JsonIgnore
	private Long lastSyncTimestamp;
	private boolean deleted;
	@JsonIgnore
	private UserSettings userSettings;
}