package de.ozgcloud.user.userprofile;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;

import com.google.rpc.Code;

import de.ozgcloud.user.UserService;
import de.ozgcloud.user.UserTestFactory;
import de.ozgcloud.user.common.callcontext.CallContextInterceptorTestFactory;
import de.ozgcloud.user.grpc.userprofile.UserProfileServiceGrpc.UserProfileServiceBlockingStub;
import io.grpc.StatusRuntimeException;
import io.quarkus.grpc.GrpcClient;
import io.quarkus.test.InjectMock;
import io.quarkus.test.junit.QuarkusTest;

@QuarkusTest
class UserProfileGrpcServiceITCase {

	@GrpcClient
	UserProfileServiceBlockingStub serviceStub;

	@InjectMock
	UserService userService;

	@DisplayName("Get by id")
	@Nested
	class TestGetById {

		private GrpcGetUserProfileRequest request = GrpcGetUserProfileRequest.newBuilder().setUserId("73").build();

		@Test
		void shouldReturnUserProfile() {
			when(userService.getById(anyString())).thenReturn(UserTestFactory.create());

			var response = serviceStub.withInterceptors(CallContextInterceptorTestFactory.createClientInterceptorWithMetadata()).getById(request);

			assertThat(response.getUserProfile().getFirstName()).isEqualTo(UserTestFactory.FIRST_NAME);
		}

		@Test
		void shouldReturnStatusRuntimeExceptionOnMissingClientName() {
			var preparedServiceStub = serviceStub.withInterceptors(CallContextInterceptorTestFactory.createClientInterceptor());

			assertThatThrownBy(() -> preparedServiceStub.getById(request))
					.isInstanceOf(StatusRuntimeException.class)
					.hasMessageContaining("Missing client name in grpc header for request")
					.extracting(ex -> ((StatusRuntimeException) ex).getStatus().getCode().value()).isEqualTo(Code.PERMISSION_DENIED_VALUE);
		}
	}
}